<?php

/*
Written by Chris Jean for iThemes.com
Version 2.0.0

Version History
	1.0.0 - 2009-12-11
		Initial release version
	1.0.1 - 2009-12-15
		Updated add_extension to only output
			if extension is set
	1.0.2 - 2009-12-16
		Fixed bug where core Builder style.css was recognized
			as a Layout Style
	1.0.3 - 2009-12018
		Fixed warnings generated by add_extension
	2.0.0 - 2010-01-07
		Changed from layout-styles.php to extensions.php
		Changed all instances of layout_styles to extensions
*/


if ( ! class_exists( 'BuilderExtensions' ) ) {
	class BuilderExtensions {
		var $_extension_directory = 'extensions';
		
		function BuilderExtensions() {
			add_action( 'builder_layout_engine_add_extension', array( &$this, 'add_extension' ) );
			add_action( 'builder_layout_engine_run_extension_functions', array( &$this, 'run_extension_functions' ) );
			
			add_filter( 'builder_layout_engine_get_extensions', array( &$this, 'get_extensions' ) );
			add_filter( 'builder_layout_engine_get_extension_data', array( &$this, 'get_extension_data' ) );
		}
		
		function add_extension( $extension = null ) {
			if ( empty( $extension ) ) {
				$layout = apply_filters( 'builder_layout_engine_get_current_layout', array() );
				
				if ( isset( $layout['extension'] ) )
					$extension = $layout['extension'];
			}
			
			if ( empty( $extension ) )
				return;
			
			$extension_url = get_template_directory_uri() . '/' . $this->_extension_directory . '/' . $extension . '/style.css';
			
			echo "<link rel='stylesheet' href='$extension_url' type='text/css' media='screen' />\n";
		}
		
		function run_extension_functions( $extension ) {
			$file = get_template_directory() . '/' . $this->_extension_directory . '/' . $extension . '/functions.php';
			
			if ( file_exists( $file ) )
				ITUtility::include_file_once( $file );
		}
		
		function get_extensions( $extensions ) {
			$extension_directory = get_template_directory() . '/' . $this->_extension_directory;
			
			if ( false === ( $dir = opendir( $extension_directory ) ) )
				return $extensions;
			
			while ( false !== ( $file = readdir( $dir ) ) ) {
				if ( preg_match( '|^\.+$|', $file ) )
					continue;
				if ( ! is_file( "{$extension_directory}/$file/style.css" ) )
					continue;
				
				$extension = $this->get_extension_data( array( 'directory' => $file ) );
				
				if ( is_array( $extension ) )
					$extensions[$extension['directory']] = $extension;
			}
			
			closedir( $dir );
			
			ITUtility::require_file_once( 'it-array-sort.php', 'ITArraySort' );
			$sorter = new ITArraySort( $extensions, 'name' );
			$extensions = $sorter->get_sorted_array();
			
			return $extensions;
		}
		
		function get_extension_data( $extension_data ) {
			$extension_directory = get_template_directory() . '/' . $this->_extension_directory;
			
			$dir = $extension_data['directory'];
			
			if ( ! file_exists( "$extension_directory/$dir/style.css" ) )
				return $extension_data;
			
			$file = "$extension_directory/$dir/style.css";
			
			$file_handle = fopen( $file, 'r' );
			$data = fread( $file_handle, 8192 );
			fclose( $file_handle );
			
			preg_match( '|Name:(.*)$|mi', $data, $name );
			
			$name = trim( $name[1] );
			
			if ( empty( $name ) )
				return $extension_data;
			
			$extension_data['file'] = $file;
			$extension_data['name'] = $name;
			
			return $extension_data;
		}
	}
	
	new BuilderExtensions();
}


?>
