<?php

/*
Written by Chris Jean for iThemes.com
Version 2.0.5

Version History
	0.0.6 - 2009-06-19
		Made it_layout_engine global var
		Pulled register_layout_module action outside class to after "new BuilderLayoutEngine()" call
		Done for PHP 4 compatibility
	0.0.7 - 2009-10-16
		Updated register_sidebar code to create widget-specific classes.
	0.0.8 - 2009-12-01
		Moved requires out to init.php
	1.0.0 - 2009-12-02
		Added calls to modules' print_open_module_wrapper/print_close_module_wrapper methods
		Release ready
	1.1.0 - 2009-12-08
		Added guid to module->render arguments
		Replaced direct sidebar registration with a call to the
			it_layout_engine_register_layout_sidebars action
	1.2.0 - 2009-12-10
		Added ability to render category and tag-specific views
		Added builder-module-last class for last module
	1.2.1 - 2009-12-11
		Moved setup_layout call from template_redirect action to inside the init function. This
			allows the Layout Style functions.php files to load earlier
		Added container wrappers
		Added it_layout_engine_filter_current_layout filter
		Added it_layout_engine_run_extension_functions action call to end of setup_layout()
	1.2.2 - 2009-12-11
		Unfortunately, running setup_layout from init rather than template_redirect didn't work.
			The query variables aren't set before init, so page detection cannot work.
		setup_layout moved back to template_redirect action
	1.2.3 - 2010-01-05
		Changed get_layout_modules to builder_get_layout_modules
	2.0.0 - 2010-01-07
		Major update
		Most functions are rewritten
		Many hooks were changed
	2.0.1 - 2010-01-07
		Added wp_footer action call on the footer-handler code
	2.0.2 - 2010-01-10
		Added .builder-module-last-before-footer class to last module if no footer is present
	2.0.3 - 2010-01-10
		Added .builder-module-before-MODULE to inner module wrapper
		Added .builder-module-after-MODULE to inner module wrapper
		Added .builder-view-VIEW to inner container wrapper
	2.0.4 - 2010-01-10
		Improved footer.php loading code
		Now a child theme's footer.php can override the parent's footer.php
	2.0.5 - 2010-01-12
		Fixed get_footer and wp_footer action conditional
*/


if ( ! class_exists( 'BuilderLayoutEngine' ) ) {
	class BuilderLayoutEngine {
		var $_layout_id = '';
		var $_layout = array();
		var $_view = '';
		var $_modules = array();
		var $_options = array();
		var $_available_views = array();
		var $_var = 'layout_settings';
		
		
		function BuilderLayoutEngine() {
			$this->_storage =& new ITStorage( $this->_var );
			$this->_available_views = apply_filters( 'builder_layout_engine_get_available_views', array() );
			
			add_action( 'builder_layout_engine_render', array( &$this, 'render' ) );
			add_action( 'init', array( &$this, 'init' ) );
			add_action( 'template_redirect', array( &$this, 'setup_layout' ) );
			add_action( 'builder_layout_engine_render_container_start', array( &$this, 'render_container_start' ) );
			add_action( 'builder_layout_engine_render_container_end', array( &$this, 'render_container_end' ) );
			
			add_filter( 'builder_layout_engine_filter_container_outer_wrapper_attributes', array( &$this, 'get_container_outer_wrapper_attributes' ) );
			add_filter( 'builder_layout_engine_get_container_width', array( &$this, 'get_container_width' ), 0 );
			add_filter( 'builder_layout_engine_get_current_layout', array( &$this, 'get_current_layout' ), 0 );
		}
		
		function init() {
			$this->_modules = apply_filters( 'builder_get_modules', array() );
			$this->_layout_settings = $this->_storage->load();
		}
		
		function render( $view ) {
			$this->_view = $view;
			
			get_header();
			
			if ( file_exists( get_stylesheet_directory() . '/footer.php' ) )
				load_template( get_stylesheet_directory() . '/footer.php' );
			else if ( file_exists( get_template_directory() . '/footer.php' ) )
				load_template( get_template_directory() . '/footer.php' );
			
			do_action( 'builder_layout_engine_render_container_start' );
			
			end( $this->_layout_settings['layouts'][$this->_layout_id]['modules'] );
			$last_id = key( $this->_layout_settings['layouts'][$this->_layout_id]['modules'] );
			
			$module_positions = array();
			$module_counts = array();
			$module_count = 0;
			$footer_position = 0;
			
			foreach ( (array) $this->_layout_settings['layouts'][$this->_layout_id]['modules'] as $id => $module ) {
				$module_count++;
				
				if ( 'footer' === $module['module'] )
					$footer_position = $module_count;
				
				if ( ! isset( $module_counts[$module['module']] ) )
					$module_counts[$module['module']] = 0;
				$module_counts[$module['module']]++;
				
				$module_positions[$module_count] = $module['module'];
			}
			
			$module_use_count = array();
			$module_count = 0;
			
			$class_prefix = apply_filters( 'builder_module_filter_css_prefix', 'builder-module' );
			
			foreach ( (array) $this->_layout_settings['layouts'][$this->_layout_id]['modules'] as $id => $module ) {
				if ( method_exists( $this->_modules[$module['module']], 'render' ) ) {
					if ( ! isset( $module_use_count[$module['module']] ) )
						$module_use_count[$module['module']] = 0;
					$module_use_count[$module['module']]++;
					
					$module_count++;
					
					
					$module['id'] = $id;
					$module['layout'] = $this->_layout_settings['layouts'][$this->_layout_id]['description'];
					$module['view'] = $view;
					$module['class_prefix'] = $class_prefix;
					
					$module['outer_wrapper']['class'] = array(
						"$class_prefix-outer-wrapper",
						"$class_prefix-{$module['module']}-outer-wrapper",
					);
					
					$module['inner_wrapper']['class'] = array(
						$class_prefix,
						"$class_prefix-{$module['module']}",
						"$class_prefix-$id",
						"$class_prefix-{$module['module']}-{$module_use_count[$module['module']]}",
					);
					
					if ( $id === $last_id )
						$module['inner_wrapper']['class'][] = "$class_prefix-last";
					
					if ( $module_use_count[$module['module']] === $module_counts[$module['module']] )
						$module['inner_wrapper']['class'][] = "$class_prefix-{$module['module']}-last";
					
					if ( isset( $module_positions[$module_count + 1] ) )
						$module['inner_wrapper']['class'][] = "$class_prefix-before-{$module_positions[$module_count + 1]}";
					if ( ( $id === $last_id ) && ( 0 === $footer_position ) )
						$module['inner_wrapper']['class'][] = "$class_prefix-before-footer";
					if ( $module_count > 1 )
						$module['inner_wrapper']['class'][] = "$class_prefix-after-{$module_positions[$module_count - 1]}";
					
					$module['inner_wrapper']['class'][] = 'clearfix';
					
					$module['inner_wrapper']['id'] = "$class_prefix-{$module['guid']}";
					
					
					$this->_modules[$module['module']]->render( $module );
				}
			}
			
			do_action( 'builder_layout_engine_render_container_end' );
			
			if ( 0 === $footer_position ) {
				do_action( 'get_footer' );
				do_action( 'wp_footer' );
			}
			
			echo "\n</body>\n</html>";
		}
		
		function get_current_layout( $layout ) {
			return $this->_layout_settings['layouts'][$this->_layout_id];
		}
		
		function get_container_width( $width ) {
			if ( 'custom' === $this->_layout_settings['layouts'][$this->_layout_id]['width'] )
				$width = intval( $this->_layout_settings['layouts'][$this->_layout_id]['custom_width'] );
			else
				$width = intval( $this->_layout_settings['layouts'][$this->_layout_id]['width'] );
			
			return $width;
		}
		
		function get_container_outer_wrapper_attributes( $options ) {
			$width = apply_filters( 'builder_layout_engine_get_container_width', 0 );
			
			$options['class'][] = 'builder-container-outer-wrapper';
			
			return $options;
		}
		
		function render_container_start() {
			$view = str_replace( '.php', '', $this->_view );
			
			$outer_wrapper_attributes = apply_filters( 'builder_layout_engine_filter_container_outer_wrapper_attributes', array() );
			
			$inner_wrapper_attributes = array( 'class' => array( 'builder-container', "builder-view-$view" ), 'id' => "builder-container-{$this->_layout_settings['layouts'][$this->_layout_id]['guid']}" );
			$inner_wrapper_attributes = apply_filters( 'builder_layout_engine_filter_container_inner_wrapper_attributes', $inner_wrapper_attributes );
			
			ITUtility::print_open_tag( 'div', $outer_wrapper_attributes );
			ITUtility::print_open_tag( 'div', $inner_wrapper_attributes );
		}
		
		function render_container_end() {
			echo "\n</div>\n</div>\n";
		}
		
		function setup_layout() {
			global $post;
			
			
			if ( is_singular() && is_object( $post ) )
				$this->_layout_id = get_post_meta( $post->ID, '_custom_layout', true );
			
			if ( empty( $this->_layout_id ) || ! is_array( $this->_layout_settings['layouts'][$this->_layout_id] ) ) {
				$priority = 0;
				
				foreach ( (array) $this->_layout_settings['views'] as $function => $layout ) {
					if ( ! isset( $this->_available_views[$function]['priority'] ) || ( $this->_available_views[$function]['priority'] <= $priority ) )
						continue;
					
					$pass = false;
					
					if ( $functions = explode( '||', $function ) ) {
						foreach ( (array) $functions as $test_function )
							if ( function_exists( $test_function ) && $test_function() )
								$pass = true;
					}
					else if ( $functions = explode( '&&', $function ) ) {
						$pass = true;
						
						foreach ( (array) $functions as $test_function )
							if ( ! ( function_exists( $test_function ) && $test_function() ) )
								$pass = false;
					}
					else if ( list( $test_function, $argument ) = explode( '|:|', $function ) ) {
						if ( function_exists( $test_function ) && $test_function( $argument ) )
							$pass = true;
					}
					else {
						if ( function_exists( $function ) && $function() )
							$pass = true;
					}
					
					if ( $pass ) {
						$this->_layout_id = $layout;
						$this->_layout_function = $function;
						$priority = $this->_available_views[$function]['priority'];
					}
				}
			}
			
			global $wp_the_query;
			
			if ( isset( $wp_the_query->query_vars['cat'] ) ) {
				if ( isset( $this->_layout_settings['views']["is_category__{$wp_the_query->query_vars['cat']}"] ) )
					$this->_layout_id = $this->_layout_settings['views']["is_category__{$wp_the_query->query_vars['cat']}"];
			}
			if ( isset( $wp_the_query->query_vars['tag_id'] ) ) {
				if ( isset( $this->_layout_settings['views']["is_tag__{$wp_the_query->query_vars['tag_id']}"] ) )
					$this->_layout_id = $this->_layout_settings['views']["is_tag__{$wp_the_query->query_vars['tag_id']}"];
			}
			
			$this->_layout_id = apply_filters( 'builder_layout_engine_filter_current_layout', $this->_layout_id );
			
			if ( empty( $this->_layout_settings['layouts'][$this->_layout_id] ) )
				$this->_layout_id = $this->_layout_settings['default'];
			
			$this->_layout = $this->_layout_settings['layouts'][$this->_layout_id]['modules'];
			
			do_action( 'builder_layout_engine_register_sidebars', $this->_layout_id );
			
			if ( isset( $this->_layout_settings['layouts'][$this->_layout_id]['extension'] ) )
				do_action( 'builder_layout_engine_run_extension_functions', $this->_layout_settings['layouts'][$this->_layout_id]['extension'] );
		}
	}
	
	new BuilderLayoutEngine();
}

?>
