<?php

/*
Written by Chris Jean for iThemes.com
Version 2.2.2

Version History
	See history.txt
*/



if ( ! class_exists( 'LayoutModuleWidgetBar' ) ) {
	class LayoutModuleWidgetBar extends LayoutModule {
		var $_name = 'Widget Bar';
		var $_var = 'widget-bar';
		var $_description = 'This module can contain one to three widget locations.';
		var $_editor_width = 450;
		var $_has_sidebars = false;
		
		
		function get_layout_option() {
			return 'type';
		}
		
		function _get_defaults( $defaults ) {
			$new_defaults = array(
				'type'						=> '3',
				'name'						=> 'Widget Bar',
				'widget_percents'			=> '33.333,33.333,33.333',
				'custom_widget_percents'	=> '',
			);
			
			return ITUtility::merge_defaults( $new_defaults, $defaults );
		}
		
		function _validate( $result ) {
			if ( empty( $_POST['name'] ) )
				$result['errors'][] = 'You must supply a Name for this Widget Bar';
			
			if ( 'custom' === $_POST['widget_percents'] ) {
				$result['data']['custom_widget_percents'] = preg_replace( '/[\s%]+/', '', $_POST['custom_widget_percents'] );
				
				if ( count( explode( ',', $_POST['custom_widget_percents'] ) ) != $_POST['type'] )
					$result['errors'][] = "You must supply {$_POST['type']} custom widget width percentages separated by commas";
				else if ( preg_match( '/[^0-9,\.]/', $result['data']['custom_widget_percents'] ) ) {
					$result['errors'][] = 'Only numbers can be used for custom widget width percentages';
				}
				else if ( ( array_sum( explode( ',', $result['data']['custom_widget_percents'] ) ) > 102 ) || ( array_sum( explode( ',', $result['data']['custom_widget_percents'] ) ) < 98 ) )
					$result['errors'][] = 'The sum of the custom widget width percentages must equal 100';
				else {
					foreach ( (array) explode( ',', $result['data']['custom_widget_percents'] ) as $percent ) {
						if ( $percent < 1 ) {
							$result['errors'][] = 'The minimum percentage width for a widget area is 1%. Please ensure that all of your custom widget width percentages are 1% or greater.';
							break;
						}
					}
				}
			}
			
			return $result;
		}
		
		function _get_default_percents() {
			$default_percents = array(
				'1'		=> array(
					'100'	=> '100%',
				),
				'2'		=> array(
					'50,50'		=> '50% / 50%',
					'80,20'		=> '80% / 20%',
					'20,80'		=> '20% / 80%',
					'70,30'		=> '70% / 30%',
					'30,70'		=> '30% / 70%',
					'60,40'		=> '60% / 40%',
					'40,60'		=> '40% / 60%',
					'custom'	=> 'Custom...',
				),
				'3'		=> array(
					'33.333,33.333,33.333'	=> '33% / 33% / 33%',
					'60,20,20'				=> '60% / 20% / 20%',
					'20,60,20'				=> '20% / 60% / 20%',
					'20,20,60'				=> '20% / 20% / 60%',
					'50,25,25'				=> '50% / 25% / 25%',
					'25,50,25'				=> '25% / 50% / 25%',
					'25,25,50'				=> '25% / 25% / 50%',
					'custom'				=> 'Custom...',
				),
				'4'		=> array(
					'25,25,25,25'	=> '25% / 25% / 25% / 25%',
					'40,20,20,20'	=> '40% / 20% / 20% / 20%',
					'20,40,20,20'	=> '20% / 40% / 20% / 20%',
					'20,20,40,20'	=> '20% / 20% / 40% / 20%',
					'20,20,20,40'	=> '20% / 20% / 20% / 40%',
					'custom'	=> 'Custom...',
				),
				'5'		=> array(
					'20,20,20,20,20'	=> '20% / 20% / 20%/ 20% / 20%',
					'custom'	=> 'Custom...',
				),
			);
			
			return $default_percents;
		}
		
		function _start_table_edit( $form, $results = true ) {
			$types = array(
				'1'		=> '1',
				'2'		=> '2',
				'3'		=> '3',
				'4'		=> '4',
				'5'		=> '5',
			);
			
			if ( empty( $_REQUEST['type'] ) )
				$_REQUEST['type'] = '1';
			
			$default_percents = $this->_get_default_percents();
			
			if ( isset( $default_percents[$_REQUEST['type']] ) )
				$start_widget_percents = $default_percents[$_REQUEST['type']];
			else
				$start_widget_percents = '';
			
?>
	<tr><td><label for="name">Name</label></td>
		<td>
			<?php $form->add_text_box( 'name' ); ?><br />
			Choose a descriptive, short name for this widget bar.<br />
			The name will help you identify the widget area(s).
		</td>
	</tr>
	<tr><td><label for="type">Number&nbsp;of&nbsp;Widgets</label></td>
		<td><?php $form->add_drop_down( 'type', $types ); ?></td>
	</tr>
	<tr style="vertical-align:top;"><td>Widget Widths</td>
		<td id="widget_percents_row">
			<?php $form->add_drop_down( 'widget_percents', $start_widget_percents ); ?>
			<div id="widget-percents-custom" style="display:none;">
				<?php $form->add_text_box( 'custom_widget_percents', array( 'size' => '20', 'maxlength' => '30' ) ); ?><br />
				Specify the percentage width of each widget area.<br />
				For example, if you have three widget areas and want to separate them into 20%, 30%, and 50% widths, input <strong><code>20,30,50</code></strong> in the box above.
			</div>
		</td>
	</tr>
<?php
		
		}
		
		function _after_table_edit( $form, $results = true ) {
			$default_percents = $this->_get_default_percents();
			
?>
	<div id="percent-width-container" style="display:none;">
		<?php foreach ( (array) $default_percents as $id => $options ) : ?>
			<div id="percent-width-container-<?php echo $id; ?>">
				<?php $form->add_drop_down( "percent-width-$id", $options ); ?>
			</div>
		<?php endforeach; ?>
	</div>
	
	<script type="text/javascript">
		/* <![CDATA[ */
		init_widget_bar_editor();
		/* ]]> */
	</script>
<?php
			
		}
		
		function _render_element( $fields ) {
			$this->_render( $fields );
		}
		
		function _render( $fields ) {
			$data = $fields['data'];
			
			$data['type'] = (int) $data['type'];
			if ( ( $data['type'] < 1 ) )
				$data['type'] = 1;
			
			
			$remaining_width = $this->_widths['container_width'];
			
			if ( 'custom' === $data['widget_percents'] )
				$widths = explode( ',', $data['custom_widget_percents'] );
			else
				$widths = explode( ',', $data['widget_percents'] );
			
			for ( $count = 0; $count < $data['type']; $count++ ) {
				if ( ( $count + 1 ) == $data['type'] )
					$width = $remaining_width;
				else
					$width = ceil( $this->_widths['container_width'] * ( $widths[$count] / 100 ) );
				
				$remaining_width -= $width;
				$num = $count + 1;
				
				$outer_wrapper = array( 'class' => array( "{$fields['class_prefix']}-block-outer-wrapper" ), 'style' => array( "width:{$width}px;", 'overflow:hidden;' ) );
				
				$inner_wrapper = array( 'class' => array( "{$fields['class_prefix']}-block", "{$fields['class_prefix']}-sidebar", 'clearfix' ) );
				
				$sidebar_wrapper = array( 'class' => array( 'widget-wrapper' ) );
				
				if ( '1' == $data['type'] ) {
					$outer_wrapper['class'][] = 'single';
					$sidebar_wrapper['class'][] = 'widget-wrapper-single';
					$sidebar_wrapper['class'][] = 'single';
				}
				else {
					if ( 0 === $count ) {
						$outer_wrapper['class'][] = 'left';
						$sidebar_wrapper['class'][] = 'widget-wrapper-left';
						$sidebar_wrapper['class'][] = 'left';
					}
					else if ( $count == $data['type'] - 1 ) {
						$outer_wrapper['class'][] = 'right';
						$sidebar_wrapper['class'][] = 'widget-wrapper-right';
						$sidebar_wrapper['class'][] = 'right';
					}
					else {
						$outer_wrapper['class'][] = 'middle';
						$sidebar_wrapper['class'][] = 'widget-wrapper-middle';
						$sidebar_wrapper['class'][] = 'middle';
					}
				}
				
				$outer_wrapper['class'][] = 'clearfix';
				$sidebar_wrapper['class'][] = 'clearfix';
				
				ITUtility::print_open_tag( 'div', $outer_wrapper );
				ITUtility::print_open_tag( 'div', $inner_wrapper );
				
				$this->_render_sidebar( $fields, "{$fields['layout']} - Widget Bar - {$fields['id']} - $num", $sidebar_wrapper );
				
				echo "</div>\n";
				echo "</div>\n";
			}
		}
		
		function register_sidebars( $module_data, $id, $layout ) {
			for ( $count = 1; $count <= $module_data['data']['type']; $count++ )
				$this->_register_sidebar( "$layout - {$this->_name} - $id - $count", $module_data['guid'] );
		}
	}
	
	new LayoutModuleWidgetBar();
}

?>
