<?php

/*
Written by Chris Jean for iThemes.com
Version 2.0.0

Version History
	1.0.0 - 2009-12-02
		Release ready
	1.0.1 - 2009-12-08
		Fixed warnings
	1.1.0 - 2009-12-09
		Added code that handles hidding sidebars properly
	1.1.1 - 2009-12-09
		Fixed bug where hidden sidebars could lose widgets
		This was a ton of work. Please appreciate it.
	1.1.2 - 2009-12-10
		Added widget-title class to filler widget content h4 tag
	1.1.3 - 2009-12-11
		Sidebars only register on admin pages that need them
	1.1.4 - 2010-01-05
		Changed get_layout_modules to builder_get_layout_modules
	2.0.0 - 2010-01-07
		Changed filter builder_layout_engine_register_layout_sidebars to builder_layout_engine_register_sidebars
		Changed filter builder_get_layout_modules to builder_get_modules
		Changed function register_layout_sidebars to register_sidebars
*/


if ( ! class_exists( 'BuilderSidebars' ) ) {
	class BuilderSidebars {
		var $_modules = array();
		var $_layout_settings = array();
		
		
		function BuilderSidebars() {
			if ( empty( $_POST ) )
				add_action( 'sidebar_admin_setup', array( &$this, 'admin_register_all_sidebars' ) );
			
			add_action( 'init', array( &$this, 'init' ) );
			add_action( 'builder_layout_engine_register_sidebars', array( &$this, 'register_sidebars' ) );
			add_action( 'builder_layout_engine_register_sidebar', array( &$this, 'register_sidebar' ) );
			add_action( 'builder_layout_engine_render_sidebar', array( &$this, 'render_sidebar' ) );
			
			add_filter( 'pre_update_option_sidebars_widgets', array( &$this, 'filter_sidebars_widgets' ) );
		}
		
		function init() {
			$this->_modules = apply_filters( 'builder_get_modules', array() );
			$this->_layout_settings = apply_filters( 'it_storage_load_layout_settings', array() );
		}
		
		function filter_sidebars_widgets( $data ) {
			if ( ! defined( 'DOING_AJAX' ) || ( true !== DOING_AJAX ) || ! isset( $_REQUEST['action'] ) || ( 'widgets-order' !== $_REQUEST['action'] ) )
				return $data;
			
			$widgets = get_option( 'sidebars_widgets' );
			
			foreach ( (array) $data as $id => $entry )
				$widgets[$id] = $entry;
			
			return $widgets;
		}
		
		function admin_register_all_sidebars() {
			global $wp_registered_sidebars;
			
			foreach ( (array) $this->_layout_settings['layouts'] as $layout => $layout_data ) {
				if ( ! isset( $layout_data['hide_widgets'] ) || ( 'yes' !== $layout_data['hide_widgets'] ) )
					$this->register_sidebars( $layout );
			}
			
			$cached_sidebars = $wp_registered_sidebars;
			
			foreach ( (array) $this->_layout_settings['layouts'] as $layout => $layout_data ) {
				if ( isset( $layout_data['hide_widgets'] ) && ( 'yes' === $layout_data['hide_widgets'] ) )
					$this->register_sidebars( $layout );
			}
			
			$this->_hidden_sidebars = array();
			
			foreach ( (array) $wp_registered_sidebars as $id => $sidebar ) {
				if ( ! isset( $cached_sidebars[$id] ) )
					$this->_hidden_sidebars[] = $id;
			}
			
			add_action( 'admin_head', array( &$this, 'admin_unregister_hidden_sidebars' ) );
		}
		
		function admin_add_unregister_hidden_sidebars() {
			add_action( 'admin_head', array( &$this, 'admin_unregister_hidden_sidebars' ) );
		}
		
		function admin_unregister_hidden_sidebars() {
			foreach ( (array) $this->_hidden_sidebars as $id )
				unregister_sidebar( $id );
		}
		
		function register_sidebars( $layout ) {
			if ( isset( $this->_layout_settings['layouts'][$layout] ) ) {
				foreach ( (array) $this->_layout_settings['layouts'][$layout]['modules'] as $id => $module ) {
					if ( method_exists( $this->_modules[$module['module']], 'register_sidebars' ) )
						$this->_modules[$module['module']]->register_sidebars( $module, $id, $this->_layout_settings['layouts'][$layout]['description'] );
				}
			}
		}
		
		function register_sidebar( $name ) {
			$options = array(
				'before_widget'		=> '<div class="widget %2$s" id="%1$s">',
				'after_widget'		=> '</div>',
				'before_title'		=> '<h4 class="widget-title">',
				'after_title'		=> '</h4>'
			);
			$options = apply_filters( 'builder_filter_register_sidebar_options', $options );
			
			if ( is_array( $name ) )
				$options = array_merge( $options, $name );
			else
				$options['name'] = $name;
			
			if ( empty( $options['id'] ) )
				$options['id'] = str_replace( ' ', '', preg_replace( '/(\w)\s(\w)/', '$1_$2', strtolower( $options['name'] ) ) );
			
			register_sidebar( $options );
		}
		
		function render_sidebar( $options ) {
			global $wp_theme_options;
			
?>
	<?php if ( ! dynamic_sidebar( $options['sidebar_name'] ) ) : ?>
		<?php if ( 'yes' === $wp_theme_options['identify_widget_areas'] ) : ?>
			<div class="widget">
				<h4 class="widget-title">This is a Widget Section</h4>
				<p>This section is widgetized. If you would like to add content to this section, you may do so by using the Widgets panel from within your WordPress Admin Dashboard. This Widget Section is called "<?php echo $options['sidebar_name']; ?>"</p>
			</div>
		<?php endif; ?>
	<?php endif; ?>
<?php
			
		}
	}
	
	new BuilderSidebars();
}


?>
