<?php

/*
Written by Chris Jean for iThemes.com
Version 2.0.0

Version History
	1.0.0 - 2009-12-07
		Release ready
	1.0.1 - 2009-12-17
		Removed uniqid warning caused by PHP 4
	2.0.0 - 2010-01-07
		Added upgrade path for 1.2
*/

if ( ! class_exists( 'BuilderLayoutStorageUpgrade' ) ) {
	class BuilderLayoutStorageUpgrade {
		var $_var = 'layout_settings';
		
		
		function BuilderLayoutStorageUpgrade() {
			add_filter( "it_storage_upgrade_{$this->_var}", array( &$this, 'do_upgrade' ) );
		}
		
		function do_upgrade( $upgrade_data ) {
			$data = $upgrade_data['data'];
			$current_version = $upgrade_data['current_version'];
			
			
			$data = $this->_upgrade_to_1_1( $data, $current_version );
			$data = $this->_upgrade_to_1_2( $data, $current_version );
			
			
			$upgrade_data['data'] = $data;
			
			return $upgrade_data;
		}
		
		function _upgrade_to_1_1( $data, $current_version ) {
			foreach ( (array) $data['layouts'] as $id => $layout ) {
				foreach ( (array) $layout['modules'] as $module_id => $module )
					if ( empty( $module['guid'] ) )
						$data['layouts'][$id]['modules'][$module_id]['guid'] = uniqid( '' );
			}
			
			foreach ( (array) $data['layouts'] as $id => $layout ) {
				if ( empty( $layout['guid'] ) ) {
					global $wpdb;
					
					
					$new_id = uniqid( '' );
					
					$layout['guid'] = $new_id;
					
					unset( $data['layouts'][$id] );
					$data['layouts'][$new_id] = $layout;
					
					if ( $id === $data['default'] )
						$data['default'] = $new_id;
					
					foreach ( (array) $data['views'] as $view_id => $layout_id ) {
						if ( $id === $layout_id )
							$data['views'][$view_id] = $new_id;
					}
					
					
					$wpdb->query( $wpdb->prepare( "UPDATE {$wpdb->postmeta} SET meta_value='$new_id' WHERE meta_key='_custom_layout' AND meta_value=%s", $id ) );
				}
			}
			
			if ( version_compare( $data['storage_version'], $current_version, '<' ) )
				$data['storage_version'] = '1.1';
			
			return $data;
		}
		
		function _upgrade_to_1_2( $data, $current_version ) {
			foreach ( (array) $data['layouts'] as $id => $layout ) {
				if ( isset( $layout['layout_style'] ) ) {
					$data['layouts'][$id]['extension'] = $layout['layout_style'];
					unset( $data['layouts'][$id]['layout_style'] );
				}
			}
			
			if ( version_compare( $data['storage_version'], $current_version, '<' ) )
				$data['storage_version'] = '1.2';
			
			return $data;
		}
	}
	
	new BuilderLayoutStorageUpgrade();
}
